import * as React from 'react';
import {getMimeTypeDescription, VideoFile} from '../../utils/blob';
import {formatFileSize, formatNumeral} from '../../utils/numeral';

export interface VideoFilePreviewProps {
  file: VideoFile;
}

export const VideoFilePreview = React.forwardRef<HTMLVideoElement, VideoFilePreviewProps>(({
  file: f,
}, forwardedRef) => {
  const defaultRef = React.useRef<HTMLVideoElement>(null);
  const mediaControllerRef = forwardedRef ?? defaultRef;
  const seekRef = React.useRef<HTMLInputElement>(null);
  const volumeRef = React.useRef<HTMLInputElement>(null);
  const [isPlaying, setIsPlaying] = React.useState(false);

  React.useEffect(() => {
    if (typeof mediaControllerRef !== 'object') {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }

    if (isPlaying) {
      void mediaControllerRef.current.play();
      return
    }

    mediaControllerRef.current.pause();
  }, [isPlaying, mediaControllerRef]);

  React.useEffect(() => {
    if (typeof mediaControllerRef !== 'object') {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }

    if (!seekRef.current) {
      return;
    }

    const { current: mediaController } = mediaControllerRef;
    const { current: seek } = seekRef;
    seek.max = String(mediaController.duration);
    seek.value = String(mediaController.currentTime);
  }, [f, mediaControllerRef]);

  React.useEffect(() => {
    if (!volumeRef.current) {
      return;
    }

    if (typeof mediaControllerRef !== 'object') {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }

    const { current: mediaController } = mediaControllerRef;
    const { current: volume } = volumeRef;
    volume.value = String(mediaController.volume);
  }, [f, mediaControllerRef]);

  const playMedia = () => {
    setIsPlaying((p) => !p);
  };

  const seekMedia = () => {

  };

  const resetVideo: React.ReactEventHandler<HTMLVideoElement> = (e) => {
    const videoElement = e.currentTarget;
    setIsPlaying(false);
    videoElement.currentTime = 0;
  };

  const updateSeekFromPlayback: React.ReactEventHandler<HTMLVideoElement> = (e) => {
    if (!seekRef.current) {
      return;
    }

    const { current: seek } = seekRef;
    const videoElement = e.currentTarget;
    seek.value = String(videoElement.currentTime);
  };

  const adjustVolume: React.ChangeEventHandler<HTMLInputElement> = (e) => {
    if (typeof mediaControllerRef !== 'object') {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }
    const { value } = e.currentTarget;
    mediaControllerRef.current.volume = Number(value);
  };

  return (
    <div className="flex gap-4 w-full h-full relative">
      <div className={`h-full w-1/3 flex-shrink-0`}>
        {
          typeof f.metadata?.previewUrl === 'string'
          && (
            <div
              className="w-full h-full bg-black flex flex-col items-stretch"
              data-testid="preview"
            >
              <div className="w-full flex-auto relative">
              <video
                className="absolute w-full h-full top-0 left-0 block w-full h-full object-center object-contain flex-auto"
                ref={mediaControllerRef as React.RefObject<HTMLVideoElement>}
                onEnded={resetVideo}
                onTimeUpdate={updateSeekFromPlayback}
              >
                <source
                  src={f.metadata.previewUrl}
                  type={f.type}
                />
              </video>
              </div>
              <div className="w-full flex-shrink-0 h-10 flex">
                <button
                  onClick={playMedia}
                  className="w-10 h-full"
                >
                  {isPlaying ? '⏸' : '▶'}
                </button>
                <input
                  type="range"
                  className="flex-auto"
                  ref={seekRef}
                  onChange={seekMedia}
                  defaultValue="0"
                />
                <input
                  type="range"
                  ref={volumeRef}
                  max={1}
                  min={0}
                  onChange={adjustVolume}
                  step="any"
                  defaultValue="1"
                />
              </div>
            </div>
          )
        }
      </div>
      <dl className="w-2/3 flex-shrink-0 m-0" data-testid="infoBox">
        <div className="w-full">
          <dt className="sr-only">
            Name
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={f.name}
          >
            {f.name}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Type
          </dt>
          <dd
            title={f.type}
            className="m-0 w-full text-ellipsis overflow-hidden"
          >
            {getMimeTypeDescription(f.type, f.name)}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Size
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={`${formatNumeral(f.size ?? 0)} bytes`}
          >
            {formatFileSize(f.size)}
          </dd>
        </div>
        {
          typeof f.metadata?.width === 'number'
          && typeof f.metadata?.height === 'number'
          && (
            <div>
              <dt className="sr-only">
                Pixel Dimensions
              </dt>
              <dd
                className="m-0 w-full text-ellipsis overflow-hidden"
              >
                {formatNumeral(f.metadata.width)}&times;{formatNumeral(f.metadata.height)} pixels
              </dd>
            </div>
          )
        }
      </dl>
    </div>
  );
});

VideoFilePreview.displayName = 'VideoFilePreview';
